﻿# This program is free software. It comes without any warranty, to the
# extent permitted by applicable law. You can redistribute it and/or
# modify it to your heart's content, just please include this header.
# Gordon Price
# Pragmatic Praxis
# www.pragmaticpraxis.com

$uninstall32 = 'HKEY_LOCAL_MACHINE\Software\Wow6432Node\Microsoft\Windows\CurrentVersion\Uninstall'
$uninstall64 = 'HKEY_LOCAL_MACHINE\Software\Microsoft\Windows\CurrentVersion\Uninstall'
$uninstallKeys = (Get-ChildItem "Registry::$uninstall64") + 
                 (Get-ChildItem "Registry::$uninstall32")

$uninstalls = @{}
foreach ($uninstallKey in $uninstallKeys) {
    $keyPath = "Registry::$($uninstallKey.name)"
    $publisher = (Get-ItemProperty -path:$keyPath -name:'Publisher' -errorAction:silentlyContinue).Publisher
    $URLInfo = (Get-ItemProperty -path:$keyPath -name:'URLInfoAbout' -errorAction:silentlyContinue).URLInfoAbout

    if (($publisher -eq 'Dynamo') -or ($URLInfo -like '*dynamobim*')) {
        $uData = @{
            pKey = $null
            sKey = $null
            tKey = $null
            uExecutable = $null
            uArguments = $null
            iLocation = $null
        }
        $dName = (Get-ItemProperty -path:$keyPath -name:'DisplayName' -errorAction:silentlyContinue).DisplayName
        $uData.pKey = $keyPath
        if (Test-Path "Registry::$uninstall64\$($dName.subString(0, $dName.length-2))") {
            $uData.skey = "Registry::$uninstall64\$($dName.subString(0, $dName.length-2))"
        }
        if (Test-Path "Registry::$uninstall32\$dName") {
            $uData.tkey = "Registry::$uninstall32\$dName"
        }
        if (-not ($uData.iLocation = (Get-ItemProperty -path:$uData.pKey -name:'InstallLocation' -errorAction:silentlyContinue).InstallLocation)) {
            $uData.iLocation = (Get-ItemProperty -path:$uData.sKey -name:'InstallLocation' -errorAction:silentlyContinue).InstallLocation
        }
        
        $quietUninstallString = (Get-ItemProperty -path:$uData.pKey -name:'QuietUninstallString' -errorAction:silentlyContinue).QuietUninstallString
        if ($uData.pKey -match '(?<guid>\{[0-9a-fA-F]{8}\-[0-9a-fA-F]{4}\-[0-9a-fA-F]{4}\-[0-9a-fA-F]{4}\-[0-9a-fA-F]{12}\})$') {
            $uData.uExecutable = 'MsiExec.exe'
            $uData.uArguments = "/x $($matches['guid']) /q"
        } elseif ($quietUninstallString -match '"(?<executable>[^"]+)"\s+(?<arguments>.+)') {
            $uData.uExecutable = $matches['executable']
            $uData.uArguments = $matches['arguments']
        }

        if ($uData.uExecutable -and $uData.uArguments) {
            $uninstalls.Add($dName, $uData)
        } else {
            Write-Host "Error: Unable to define uninstall for $dName"
        }
    }
}

if ($uninstalls.count -gt 0) {
    Write-host "`nUninstalling Dynamo...`n"
    
    # Build Special Folders array
    $specialFolders = New-Object collections.arrayList
    foreach ($specialFolderName in [Enum]::GetNames([Environment+SpecialFolder])) {
        if ((($specialFolderPath = [environment]::getfolderpath($specialFolderName)) -ne '') -and ($specialFolders -notContains $specialFolderPath)) {
            [void]$specialFolders.Add($specialFolderPath)
        }
    }

    # Sort Uninstalls
    $sortedUninstalls = New-Object collections.specialized.orderedDictionary
    foreach ($key in $uninstalls.keys | Sort-Object) {
        $sortedUninstalls[$key] = $uninstalls[$key]
    }

    # Process Uninstalls Loop
    foreach ($uninstall in $sortedUninstalls.keys) {
        
        # Refresh Uninstall Data
        $uData = @{}
        $uData.Add('dName', $uninstall)     
        foreach ($key in $sortedUninstalls.$uninstall.keys) {
            $uData.Add($key, $sortedUninstalls.$uninstall[$key])
        }
        
        # Process Uninstall
        $process = (Start-Process -filePath:$uData.uExecutable -argumentList:$uData.uArguments -passThru -wait)
        if ($process.exitCode -eq 0) {
            $uninstallSucceeded = $true
            Write-Host "Uninstalled $($uData.dName)"
            Start-Sleep -seconds:10

            if ($uData.pKey -and (Test-Path $uData.pKey)) {
                try {
                    Remove-Item $uData.pKey -force -errorAction:stop
                    Write-Host "  Deleted Primary Key: $($uData.pKey)"
                } catch {
                    Write-host "  Delete key failed: $($_.Exception.Message)"
                }
            }
            if ($uData.sKey -and (Test-Path $uData.sKey)) {
                try {
                    Remove-Item $uData.sKey -force -errorAction:stop
                    Write-Host "  Deleted Secondary Key: $($uData.sKey)"
                } catch {
                    Write-host "  Delete key failed: $($_.Exception.Message)"
                }
            }
            if ($uData.tkey -and (Test-Path $uData.tkey)) {
                try {
                    Remove-Item $uData.tkey -force -errorAction:stop
                    Write-Host "  Deleted Tertiary key: $($uData.tkey)"
                } catch {
                    Write-host "  Delete key failed: $($_.Exception.Message)"
                }
            }
            if ($uData.iLocation -and (Test-Path $uData.iLocation)) {
                try {
                    Remove-Item "$($uData.iLocation)\*" -recurse -force -errorAction:stop
                    Write-Host "  Deleted Install Location contents: $($uData.iLocation)"
                } catch {
                    Write-host "  Delete folder contents failed: $($_.Exception.Message)"
                }
                $candidateFolderToDelete = $uData.iLocation
                :deleteEmptyFolders while ((Get-ChildItem -path:$candidateFolderToDelete -force -errorAction:silentlyContinue | Measure-Object).count -eq 0) {
                    try {
                        Remove-Item $candidateFolderToDelete -force -errorAction:stop
                        Write-host "  Deleted Install Location folder: $candidateFolderToDelete"
                        $candidateFolderToDelete = Split-Path $candidateFolderToDelete -parent
                        if ($SpecialFolders -contains $candidateFolderToDelete) {
                            break :deleteEmptyFolders
                        }
                    } catch {
                        Write-host "  Delete folder failed: $($_.Exception.Message)"
                        break :deleteEmptyFolders
                    }
                }
            }
        } else {
            Write-Host "Failed to uninstall $($uData.dName) with exit code $($process.exitCode)"
            $uninstallFailed = $true
        }
    }

    # Orphans
    if ($uninstallSucceeded -and (-not $uninstallFailed)) {
        $orphans = @('C:\Program Files\Dynamo', 'C:\Program Files\Dynamo 0.9', 'HKLM:\SOFTWARE\Microsoft\Windows\CurrentVersion\Uninstall\Dynamo 0.9')
        $orphanHeader = "`nDeleted orphan(s)"
        foreach ($orphan in $orphans) {
            if (Test-Path $orphan) {
                try {
                    Remove-Item $orphan -force -recurse -errorAction:Stop
                    if ($orphanHeader) {
                        Write-Host $orphanHeader
                        $orphanHeader = $null
                    }
                    Write-Host "  Deleted $orphan"
                } catch {
                    if ($orphanHeader) {
                        Write-Host $orphanHeader
                        $orphanHeader = $null
                    }
                    Write-host "   Delete folder failed: $($_.Exception.Message)"
                }
            }
        }
    }
} else {
    Write-host "`nNo Dynamo installed on this machine."
}
Write-host "`nCOMPLETE!"